  using System;
  using System.Collections.Generic;
  using System.Linq;
  using Michigan.Ods.Sdk.Helpers;
  using Michigan.Ods.Sdk.Models;
  using RestSharp;
  using System.Net;
  using System.Text;
  
  namespace Michigan.Ods.Sdk.Api {
    public class StudentClassGroupAssociationsApi {

      private readonly IRestClient client;

      public StudentClassGroupAssociationsApi(IRestClient client)
      {
            this.client = client;
      }
      
      /// <summary>
      /// Retrieves resources based with paging capabilities (using the &quot;Get All&quot; pattern). This GET operation provides access to resources using the &quot;Get All&quot; pattern. In this version of the API there is support for paging.
      /// </summary>
      /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
      /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
      /// <returns></returns>
      public IRestResponse<List<StudentClassGroupAssociation>> GetStudentClassGroupAssociationsAll(int? offset, int? limit) {
        var request = new RestRequest("/studentClassGroupAssociations", Method.GET);
        request.RequestFormat = DataFormat.Json;

        if (offset != null){
          request.AddParameter("offset", offset);
		}
        if (limit != null){
          request.AddParameter("limit", limit);
		}
        var response = client.Execute<List<StudentClassGroupAssociation>>(request);

        return response;
      }
      /// <summary>
      /// Retrieves resources matching values of an example resource (using the &quot;Get By Example&quot; pattern). This GET operation provides access to resources using the &quot;Get by Example&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
      /// </summary>
      /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
      /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
      /// <param name="studentUniqueId">A unique alpha-numeric code assigned to a student.</param>
      /// <param name="communityProviderLocationId">A unique number or alphanumeric code assigned to an education organization by a school, school system, a state, or other agency or entity.</param>
      /// <param name="communityProviderId">A unique number or alphanumeric code assigned to an education organization by a school, school system, a state, or other agency or entity.</param>
      /// <param name="classGroupName">The name of the class/group of students.</param>
      /// <param name="beginDate">The month, day, and year on which the student began participating in a class/group.</param>
      /// <param name="endDate">The month, day, and year on which the student stopped participating in a class/group.  </param>
      /// <param name="reasonExitedDescriptor">The reason the student stopped participating in a class/group.  </param>
      /// <param name="id"></param>
      /// <returns></returns>
      public IRestResponse<List<StudentClassGroupAssociation>> GetStudentClassGroupAssociationsAll(int? offset, int? limit, string studentUniqueId, int? communityProviderLocationId, int? communityProviderId, string classGroupName, DateTime? beginDate, DateTime? endDate, string reasonExitedDescriptor, string id) {
        var request = new RestRequest("/studentClassGroupAssociations", Method.GET);
        request.RequestFormat = DataFormat.Json;

        if (offset != null){
          request.AddParameter("offset", offset);
		}
        if (limit != null){
          request.AddParameter("limit", limit);
		}
        if (studentUniqueId != null){
          request.AddParameter("studentUniqueId", studentUniqueId);
		}
        if (communityProviderLocationId != null){
          request.AddParameter("communityProviderLocationId", communityProviderLocationId);
		}
        if (communityProviderId != null){
          request.AddParameter("communityProviderId", communityProviderId);
		}
        if (classGroupName != null){
          request.AddParameter("classGroupName", classGroupName);
		}
        if (beginDate != null){
          request.AddParameter("beginDate", beginDate);
		}
        if (endDate != null){
          request.AddParameter("endDate", endDate);
		}
        if (reasonExitedDescriptor != null){
          request.AddParameter("reasonExitedDescriptor", reasonExitedDescriptor);
		}
        if (id != null){
          request.AddParameter("id", id);
		}
        var response = client.Execute<List<StudentClassGroupAssociation>>(request);

        return response;
      }
      /// <summary>
      /// Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
      /// </summary>
      /// <param name="studentUniqueId">A unique alpha-numeric code assigned to a student.</param>
      /// <param name="communityProviderLocationId">A unique number or alphanumeric code assigned to an education organization by a school, school system, a state, or other agency or entity.</param>
      /// <param name="communityProviderId">A unique number or alphanumeric code assigned to an education organization by a school, school system, a state, or other agency or entity.</param>
      /// <param name="classGroupName">The name of the class/group of students.</param>
      /// <param name="beginDate">The month, day, and year on which the student began participating in a class/group.</param>
      /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
      /// <returns></returns>
      public IRestResponse<StudentClassGroupAssociation> GetStudentClassGroupAssociationByKey(string studentUniqueId, int? communityProviderLocationId, int? communityProviderId, string classGroupName, DateTime? beginDate, string IfNoneMatch) {
        var request = new RestRequest("/studentClassGroupAssociations", Method.GET);
        request.RequestFormat = DataFormat.Json;

        // verify required params are set
        if (studentUniqueId == null || communityProviderLocationId == null || communityProviderId == null || classGroupName == null || beginDate == null ) {
           throw new ArgumentException("API method call is missing required parameters");
        }
        if (studentUniqueId != null){
          request.AddParameter("studentUniqueId", studentUniqueId);
		}
        if (communityProviderLocationId != null){
          request.AddParameter("communityProviderLocationId", communityProviderLocationId);
		}
        if (communityProviderId != null){
          request.AddParameter("communityProviderId", communityProviderId);
		}
        if (classGroupName != null){
          request.AddParameter("classGroupName", classGroupName);
		}
        if (beginDate != null){
          request.AddParameter("beginDate", beginDate);
		}
        request.AddHeader("If-None-Match", IfNoneMatch);
        var response = client.Execute<StudentClassGroupAssociation>(request);

        return response;
      }
      /// <summary>
      /// Creates or updates resources based on the natural key values of the supplied resource. The POST operation can be used to create or update resources. In database terms, this is often referred to as an &quot;upsert&quot; operation (insert + update).  Clients should NOT include the resource &quot;id&quot; in the JSON body because it will result in an error (you must use a PUT operation to update a resource by &quot;id&quot;). The web service will identify whether the resource already exists based on the natural key values provided, and update or create the resource appropriately.
      /// </summary>
      /// <param name="body">The JSON representation of the &quot;studentClassGroupAssociation&quot; resource to be created or updated.</param>
      /// <returns></returns>
      public IRestResponse PostStudentClassGroupAssociations(StudentClassGroupAssociation body) {
        var request = new RestRequest("/studentClassGroupAssociations", Method.POST);
        request.RequestFormat = DataFormat.Json;

        // verify required params are set
        if (body == null ) {
           throw new ArgumentException("API method call is missing required parameters");
        }
        request.AddBody(body);
        var response = client.Execute(request);

        var location = response.Headers.FirstOrDefault(x => x.Name == "Location");

        if (location != null && !string.IsNullOrWhiteSpace(location.Value.ToString()))
          {
              body.id = location.Value.ToString().Split('/').Last();
          }
        return response;
      }
      /// <summary>
      /// Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
      /// </summary>
      /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
      /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
      /// <returns></returns>
      public IRestResponse<StudentClassGroupAssociation> GetStudentClassGroupAssociationsById(string id, string IfNoneMatch) {
        var request = new RestRequest("/studentClassGroupAssociations/{id}", Method.GET);
        request.RequestFormat = DataFormat.Json;

        request.AddUrlSegment("id", id);
        // verify required params are set
        if (id == null ) {
           throw new ArgumentException("API method call is missing required parameters");
        }
        request.AddHeader("If-None-Match", IfNoneMatch);
        var response = client.Execute<StudentClassGroupAssociation>(request);

        return response;
      }
      /// <summary>
      /// Updates or creates a resource based on the resource identifier. The PUT operation is used to update or create a resource by identifier.  If the resource doesn't exist, the resource will be created using that identifier.  Additionally, natural key values cannot be changed using this operation, and will not be modified in the database.  If the resource &quot;id&quot; is provided in the JSON body, it will be ignored as well.
      /// </summary>
      /// <param name="id">A resource identifier specifying the resource to be updated.</param>
      /// <param name="IfMatch">The ETag header value used to prevent the PUT from updating a resource modified by another consumer.</param>
      /// <param name="body">The JSON representation of the &quot;studentClassGroupAssociation&quot; resource to be updated.</param>
      /// <returns></returns>
      public IRestResponse PutStudentClassGroupAssociation(string id, string IfMatch, StudentClassGroupAssociation body) {
        var request = new RestRequest("/studentClassGroupAssociations/{id}", Method.PUT);
        request.RequestFormat = DataFormat.Json;

        request.AddUrlSegment("id", id);
        // verify required params are set
        if (id == null || body == null ) {
           throw new ArgumentException("API method call is missing required parameters");
        }
        request.AddHeader("If-Match", IfMatch);
        request.AddBody(body);
        var response = client.Execute(request);

        var location = response.Headers.FirstOrDefault(x => x.Name == "Location");

        if (location != null && !string.IsNullOrWhiteSpace(location.Value.ToString()))
          {
              body.id = location.Value.ToString().Split('/').Last();
          }
        return response;
      }
      /// <summary>
      /// Deletes an existing resource using the resource identifier. The DELETE operation is used to delete an existing resource by identifier.  If the resource doesn't exist, an error will result (the resource will not be found).
      /// </summary>
      /// <param name="id">A resource identifier specifying the resource to be deleted.</param>
      /// <param name="IfMatch">The ETag header value used to prevent the DELETE from removing a resource modified by another consumer.</param>
      /// <returns></returns>
      public IRestResponse DeleteStudentClassGroupAssociationById(string id, string IfMatch) {
        var request = new RestRequest("/studentClassGroupAssociations/{id}", Method.DELETE);
        request.RequestFormat = DataFormat.Json;

        request.AddUrlSegment("id", id);
        // verify required params are set
        if (id == null ) {
           throw new ArgumentException("API method call is missing required parameters");
        }
        request.AddHeader("If-Match", IfMatch);
        var response = client.Execute(request);

        return response;
      }
      /// <summary>
        ///Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
        ///</summary>
        /// <param name="studentUniqueId">A unique alpha-numeric code assigned to a student.</param>
        /// <param name="communityProviderLocationId">A unique number or alphanumeric code assigned to an education organization by a school, school system, a state, or other agency or entity.</param>
        /// <param name="communityProviderId">A unique number or alphanumeric code assigned to an education organization by a school, school system, a state, or other agency or entity.</param>
        /// <param name="classGroupName">The name of the class/group of students.</param>
        /// <param name="beginDate">The month, day, and year on which the student began participating in a class/group.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        ///<returns></returns>
        public IRestResponse PatchGetStudentClassGroupAssociationByKey(string studentUniqueId, int? communityProviderLocationId, int? communityProviderId, string classGroupName, DateTime? beginDate, string IfNoneMatch, StudentClassGroupAssociation patchStudentClassGroupAssociation)
        {
            // verify required params are set
            if (studentUniqueId == null || communityProviderLocationId == null || communityProviderId == null || classGroupName == null || beginDate == null ) {
                throw new ArgumentException("API method call is missing required parameters");
            }
            var sdkUsageRecord = new SdkUsageRecord();
            var jsonSerializer = new RestSharp.Serializers.JsonSerializer();
            var statisticsTracker = new StatisticsTracker();

            var getResponse = this.GetStudentClassGroupAssociationByKey( studentUniqueId, communityProviderLocationId, communityProviderId, classGroupName, beginDate, IfNoneMatch);

            if (!(getResponse.StatusCode == HttpStatusCode.OK || getResponse.StatusCode == HttpStatusCode.Found))
                return getResponse;

            if (statisticsTracker.ShouldTrackStatistics)
            {
                sdkUsageRecord.DateTimeStamp = DateTime.Now;
                sdkUsageRecord.HttpMethod = "PatchGetStudentClassGroupAssociationByKey";
                sdkUsageRecord.Url = "/studentClassGroupAssociations";
                var serializedGetObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.GetSizeInBytes = Encoding.UTF8.GetByteCount(serializedGetObject);
                var serializedPatchObject = jsonSerializer.Serialize(patchStudentClassGroupAssociation);
                sdkUsageRecord.PatchSizeInBytes = Encoding.UTF8.GetByteCount(serializedPatchObject);
            }

            new PatchApplier().Execute(getResponse.Data, patchStudentClassGroupAssociation);

            var putResponse = PutStudentClassGroupAssociation( getResponse.Data.id, null, getResponse.Data ); 
            if (statisticsTracker.ShouldTrackStatistics)
            {
                var serializedPutObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.PutSizeInBytes = Encoding.UTF8.GetByteCount(serializedPutObject);
                statisticsTracker.TrackIfEnabled(sdkUsageRecord);
            }
            return putResponse;
        }
        /// <summary>
        ///Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        ///</summary>
        /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        ///<returns></returns>
        public IRestResponse PatchGetStudentClassGroupAssociationsById(string id, string IfNoneMatch, StudentClassGroupAssociation patchStudentClassGroupAssociation)
        {
            // verify required params are set
            if (id == null ) {
                throw new ArgumentException("API method call is missing required parameters");
            }
            var sdkUsageRecord = new SdkUsageRecord();
            var jsonSerializer = new RestSharp.Serializers.JsonSerializer();
            var statisticsTracker = new StatisticsTracker();

            var getResponse = this.GetStudentClassGroupAssociationsById( id, IfNoneMatch);

            if (!(getResponse.StatusCode == HttpStatusCode.OK || getResponse.StatusCode == HttpStatusCode.Found))
                return getResponse;

            if (statisticsTracker.ShouldTrackStatistics)
            {
                sdkUsageRecord.DateTimeStamp = DateTime.Now;
                sdkUsageRecord.HttpMethod = "PatchGetStudentClassGroupAssociationsById";
                sdkUsageRecord.Url = "/studentClassGroupAssociations/{id}";
                var serializedGetObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.GetSizeInBytes = Encoding.UTF8.GetByteCount(serializedGetObject);
                var serializedPatchObject = jsonSerializer.Serialize(patchStudentClassGroupAssociation);
                sdkUsageRecord.PatchSizeInBytes = Encoding.UTF8.GetByteCount(serializedPatchObject);
            }

            new PatchApplier().Execute(getResponse.Data, patchStudentClassGroupAssociation);

            var putResponse = PutStudentClassGroupAssociation( getResponse.Data.id, null, getResponse.Data ); 
            if (statisticsTracker.ShouldTrackStatistics)
            {
                var serializedPutObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.PutSizeInBytes = Encoding.UTF8.GetByteCount(serializedPutObject);
                statisticsTracker.TrackIfEnabled(sdkUsageRecord);
            }
            return putResponse;
        }
        }
}
