  
using System;
using System.Collections.Generic;
using System.Linq;
using Michigan.Ods.Sdk.Helpers;
using Michigan.Ods.Sdk.Models;
using Michigan.Ods.Sdk.Models.Composites.Library;
using RestSharp;

namespace Michigan.Ods.Sdk.Api.Composites
{
    public class LibraryApi
    {
        private readonly StudentsApi _studentsApi;
        private readonly StudentSchoolAssociationsApi _studentSchoolAssociationsApi;
        private readonly StudentSectionAssociationsApi _studentSectionAssociationsApi;
        private readonly StudentLibraryServicesApi _studentLibraryServicesApi;
        private readonly StaffsApi _staffApi;

        private readonly StaffEducationOrganizationEmploymentAssociationsApi
            _staffEducationOrganizationEmploymentAssociationsApi;

        private readonly StaffLibraryServicesApi _staffLibraryServicesApi;

        public LibraryApi(IRestClient client)
        {
            _studentsApi = new StudentsApi(client);
            _studentSchoolAssociationsApi = new StudentSchoolAssociationsApi(client);
            _studentSectionAssociationsApi = new StudentSectionAssociationsApi(client);
            _studentLibraryServicesApi = new StudentLibraryServicesApi(client);
            _staffApi = new StaffsApi(client);
            _staffEducationOrganizationEmploymentAssociationsApi =
                new StaffEducationOrganizationEmploymentAssociationsApi(client);
            _staffLibraryServicesApi = new StaffLibraryServicesApi(client);
        }

        /// <summary>
        ///     Retrieves resources based with paging capabilities (using the &quot;Get All&quot; pattern). This GET operation
        ///     provides access to resources using the &quot;Get All&quot; pattern. In this version of the API there is support for
        ///     paging.
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="fields">
        ///     Specifies a subset of properties that should be returned for each entity (e.g. &quot;
        ///     property1,collection1(collProp1,collProp2)&quot;).
        /// </param>
        /// <returns></returns>
        public IRestResponse<List<LibraryStudent>> GetLibraryStudentsAll(int? offset, int? limit, string fields)
        {
            var studentGetAllResponse = _studentsApi.GetStudentsAll(offset, limit);

            if (!ManualCompositesHelper.IsValidRestResponse(studentGetAllResponse))
                return ManualCompositesHelper.CopyRestResponse(studentGetAllResponse, new List<LibraryStudent>());

            return GetManualLibraryStudentComposite(studentGetAllResponse.Data, studentGetAllResponse);
        }

        /// <summary>
        ///     Retrieves a specific resource using the resource's identifier (using the &quot;Get By Example&quot; pattern). This
        ///     GET operation provides access to resources using the &quot;Get By Example&quot; pattern. The values of any
        ///     properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="fields">
        ///     Specifies a subset of properties that should be returned for each entity (e.g. &quot;
        ///     property1,collection1(collProp1,collProp2)&quot;).
        /// </param>
        /// <param name="q">
        ///     Specifies a query filter expression for the request.  Currently only supports range-based queries on
        ///     dates and numbers (e.g. &quot;[2016-03-07..2016-03-10]&quot;).
        /// </param>
        /// <param name="studentUniqueId">A unique alphanumeric code assigned to a student.</param>
        /// <param name="citizenshipStatusType">
        ///     A unique identifier used as Primary Key, not derived from business logic, when
        ///     acting as Foreign Key, references the parent table.
        /// </param>
        /// <param name="birthCountryDescriptor">
        ///     A unique identifier used as Primary Key, not derived from business logic, when
        ///     acting as Foreign Key, references the parent table.
        /// </param>
        /// <param name="limitedEnglishProficiencyDescriptor">
        ///     An indication that the student has sufficient difficulty speaking,
        ///     reading, writing, or understanding the English language, as to require special English Language services.
        /// </param>
        /// <param name="oldEthnicityType">
        ///     Previous definition of Ethnicity combining Hispanic/latino and race:  1 - American
        ///     Indian or Alaskan Native  2 - Asian or Pacific Islander  3 - Black, not of Hispanic origin  4 - Hispanic  5 -
        ///     White, not of Hispanic origin
        /// </param>
        /// <param name="schoolFoodServicesEligibilityDescriptor">
        ///     A unique identifier used as Primary Key, not derived from
        ///     business logic, when acting as Foreign Key, references the parent table.
        /// </param>
        /// <param name="sexType">A person''s gender.</param>
        /// <param name="birthStateAbbreviationType">
        ///     The abbreviation for the name of the state (within the United States) or
        ///     extra-state jurisdiction in which an individual was born.
        /// </param>
        /// <param name="personalTitlePrefix">A prefix used to denote the title, degree, position, or seniority of the person.</param>
        /// <param name="firstName">
        ///     A name given to an individual at birth, baptism, or during another naming ceremony, or through
        ///     legal change.
        /// </param>
        /// <param name="middleName">A secondary name given to an individual at birth, baptism, or during another naming ceremony.</param>
        /// <param name="lastSurname">The name borne in common by members of a family.</param>
        /// <param name="generationCodeSuffix">
        ///     An appendage, if any, used to denote an individual's generation in his family (e.g.,
        ///     Jr., Sr., III).
        /// </param>
        /// <param name="maidenName">The person's maiden name.</param>
        /// <param name="birthDate">The month, day, and year on which an individual was born.</param>
        /// <param name="birthCity">
        ///     The set of elements that capture relevant data regarding a person's birth, including birth date
        ///     and place of birth.
        /// </param>
        /// <param name="dateEnteredUS">For students born outside of the U.S., the date the student entered the U.S.</param>
        /// <param name="multipleBirthStatus">
        ///     Indicator of whether the student was born with other siblings (i.e., twins, triplets,
        ///     etc.)
        /// </param>
        /// <param name="profileThumbnail">ProfileThumbnail.</param>
        /// <param name="hispanicLatinoEthnicity">
        ///     An indication that the individual traces his or her origin or descent to Mexico,
        ///     Puerto Rico, Cuba, Central, and South America, and other Spanish cultures, regardless of race. The term, &quot;
        ///     Spanish origin,&quot; can be used in addition to &quot;Hispanic or Latino.&quot;
        /// </param>
        /// <param name="economicDisadvantaged">
        ///     An indication of inadequate financial condition of an individual's family, as
        ///     determined by family income, number of family members/dependents, participation in public assistance programs,
        ///     and/or other characteristics considered relevant by federal, state, and local policy.
        /// </param>
        /// <param name="displacementStatus">
        ///     Indicates a state health or weather related event that displaces a group of students,
        ///     and may require additional funding, educational, or social services.
        /// </param>
        /// <param name="loginId">The login ID for the user; used for security access control interface.</param>
        /// <param name="birthInternationalProvince">
        ///     For students born outside of the US, the Province or jurisdiction in which an
        ///     individual is born.
        /// </param>
        /// <param name="id"></param>
        /// <param name="birthCountryDescriptor">
        ///     A unique identifier used as Primary Key, not derived from business logic, when
        ///     acting as Foreign Key, references the parent table.
        /// </param>
        /// <param name="yearOfEntry">
        ///     (MI-EXT) The four-digit calendar year in which the student first attended school in the
        ///     United States.
        /// </param>
        /// <param name="raceAsOfDate">(MI-EXT) The date of the most recent change in Race.</param>
        /// <param name="schoolFoodServicesEligibilityAsOfDate">
        ///     (MI-EXT) The date of the most recent change in
        ///     SchoolFoodServicesEligibility.
        /// </param>
        /// <param name="addressAsOfDate">(MI-EXT) The date of the most recent changes to Address.</param>
        /// <param name="homelessDescriptor">
        ///     A unique identifier used as Primary Key, not derived from business logic, when acting
        ///     as Foreign Key, references the parent table.
        /// </param>
        /// <param name="foodServiceStatusDescriptor">
        ///     (MI EXT) A unique identifier used as Primary Key, not derived from business
        ///     logic, when acting as Foreign Key, references the parent table.
        /// </param>
        /// <param name="medicalNotes">
        ///     (MI EXT) Medical notes such as &quot;Allergic to Bee Stings&quot; - For Transportation
        ///     records.
        /// </param>
        /// <returns></returns>
        public IRestResponse<List<LibraryStudent>> GetLibraryStudentsByExample(int? offset, int? limit,
            string fields, string q, string studentUniqueId, string citizenshipStatusType, string birthCountryDescriptor,
            string limitedEnglishProficiencyDescriptor, string oldEthnicityType,
            string schoolFoodServicesEligibilityDescriptor, string sexType, string birthStateAbbreviationType,
            string personalTitlePrefix, string firstName, string middleName, string lastSurname,
            string generationCodeSuffix, string maidenName, DateTime? birthDate, string birthCity,
            DateTime? dateEnteredUS, bool? multipleBirthStatus, string profileThumbnail, bool? hispanicLatinoEthnicity,
            bool? economicDisadvantaged, string displacementStatus, string loginId, string birthInternationalProvince,
            string id, int? yearOfEntry, DateTime? raceAsOfDate, DateTime? schoolFoodServicesEligibilityAsOfDate,
            DateTime? addressAsOfDate, string homelessDescriptor, string foodServiceStatusDescriptor,
            string medicalNotes)
        {
            var studentGetAllResponse = _studentsApi.GetStudentsAll(offset, limit, studentUniqueId, personalTitlePrefix,
                firstName, middleName, lastSurname, generationCodeSuffix, maidenName, sexType, birthDate, birthCity,
                birthStateAbbreviationType, dateEnteredUS, multipleBirthStatus, profileThumbnail,
                hispanicLatinoEthnicity, oldEthnicityType, economicDisadvantaged,
                schoolFoodServicesEligibilityDescriptor, limitedEnglishProficiencyDescriptor, displacementStatus,
                loginId, birthInternationalProvince, citizenshipStatusType, id, birthCountryDescriptor, yearOfEntry,
                raceAsOfDate, schoolFoodServicesEligibilityAsOfDate, addressAsOfDate, homelessDescriptor,
                foodServiceStatusDescriptor, medicalNotes);

            if (!ManualCompositesHelper.IsValidRestResponse(studentGetAllResponse))
                return ManualCompositesHelper.CopyRestResponse(studentGetAllResponse, new List<LibraryStudent>());

            return GetManualLibraryStudentComposite(studentGetAllResponse.Data, studentGetAllResponse);
        }

        /// <summary>
        ///     Retrieves a specific resource using the resource's identifier (using the &quot;Get By Key&quot; pattern). This GET
        ///     operation provides access to resources using the &quot;Get By Key&quot; pattern. The values of the natural key of
        ///     the resource must be fully specified, and the service will return the matching result (if it exists).
        /// </summary>
        /// <param name="studentUniqueId">A unique alphanumeric code assigned to a student.</param>
        /// <param name="fields">
        ///     Specifies a subset of properties that should be returned for each entity (e.g. &quot;
        ///     property1,collection1(collProp1,collProp2)&quot;).
        /// </param>
        /// <param name="IfNoneMatch">
        ///     The previously returned ETag header value, used here to prevent the unnecessary data transfer
        ///     of an unchanged resource.
        /// </param>
        /// <returns></returns>
        public IRestResponse<LibraryStudent> GetLibraryStudentByKey(string studentUniqueId, string fields,
            string IfNoneMatch)
        {
            var studentGetByKeyResponse = _studentsApi.GetStudentByKey(studentUniqueId, IfNoneMatch);

            if (!ManualCompositesHelper.IsValidRestResponse(studentGetByKeyResponse))
                return ManualCompositesHelper.CopyRestResponse<LibraryStudent>(studentGetByKeyResponse, null);

            var mappedResponse = GetManualLibraryStudentComposite(new List<Student> {studentGetByKeyResponse.Data},
                studentGetByKeyResponse);
            return ManualCompositesHelper.CopyRestResponse(mappedResponse, mappedResponse.Data.Single());
        }

        /// <summary>
        ///     Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET
        ///     operation provides access to resources using the &quot;Get By Id&quot; pattern.
        /// </summary>
        /// <param name="id"></param>
        /// <param name="fields">
        ///     Specifies a subset of properties that should be returned for each entity (e.g. &quot;
        ///     property1,collection1(collProp1,collProp2)&quot;).
        /// </param>
        /// <param name="IfNoneMatch">
        ///     The previously returned ETag header value, used here to prevent the unnecessary data transfer
        ///     of an unchanged resource.
        /// </param>
        /// <returns></returns>
        public IRestResponse<LibraryStudent> GetLibraryStudentsById(string id, string fields, string IfNoneMatch)
        {
            var studentsByIdResponse = _studentsApi.GetStudentsById(id, IfNoneMatch);

            if (!ManualCompositesHelper.IsValidRestResponse(studentsByIdResponse))
                return ManualCompositesHelper.CopyRestResponse<LibraryStudent>(studentsByIdResponse, null);

            var mappedResponse = GetManualLibraryStudentComposite(new List<Student> {studentsByIdResponse.Data},
                studentsByIdResponse);
            return ManualCompositesHelper.CopyRestResponse(mappedResponse, mappedResponse.Data.Single());
        }

        private IRestResponse<List<LibraryStudent>> GetManualLibraryStudentComposite(List<Student> students,
            IRestResponse originalRestResponse)
        {
            var results = new List<LibraryStudent>();
            var studentResults = students.Select(s =>
                new LibraryStudent
                {
                    studentUniqueId = s.studentUniqueId,
                    firstName = s.firstName,
                    middleName = s.middleName,
                    lastSurname = s.lastSurname,
                    sexType = s.sexType,
                    birthDate = s.birthDate,
                    otherNames = s.otherNames.Select(n =>
                        new LibraryStudent_studentOtherName
                        {
                            firstName = n.firstName,
                            middleName = n.middleName,
                            lastSurname = n.lastSurname,
                            otherNameType = n.otherNameType
                        }).ToList(),
                    addresses = s.addresses.Select(a =>
                        new LibraryStudent_studentAddress
                        {
                            addressType = a.addressType,
                            streetNumberName = a.streetNumberName,
                            apartmentRoomSuiteNumber = a.apartmentRoomSuiteNumber,
                            buildingSiteNumber = a.buildingSiteNumber,
                            city = a.city,
                            stateAbbreviationType = a.stateAbbreviationType,
                            postalCode = a.postalCode
                        }).ToList(),
                    electronicMails = s.electronicMails.Select(em =>
                        new LibraryStudent_studentElectronicMail
                        {
                            primaryEmailAddressIndicator = em.primaryEmailAddressIndicator,
                            electronicMailType = em.electronicMailType,
                            electronicMailAddress = em.electronicMailAddress
                        }).ToList(),
                    telephones = s.telephones.Select(t =>
                        new LibraryStudent_studentTelephone
                        {
                            telephoneNumberType = t.telephoneNumberType,
                            orderOfPriority = t.orderOfPriority,
                            telephoneNumber = t.telephoneNumber
                        }).ToList()
                }).ToList();
            foreach (var studentResult in studentResults)
            {
                // Get StudentSchoolAssociation records
                var getStudentSchoolAssociationsResponse =
                    _studentSchoolAssociationsApi.GetStudentSchoolAssociationsAll(
                        null, null,
                        studentResult.studentUniqueId,
                        null, null, null, null, null, null, null, null, null, null,
                        null, null, null, null, null, null, null, null, null, null);
                if (!ManualCompositesHelper.IsValidRestResponse(getStudentSchoolAssociationsResponse))
                    return ManualCompositesHelper.CopyRestResponse(getStudentSchoolAssociationsResponse, results);
                studentResult.schools =
                    getStudentSchoolAssociationsResponse.Data.Select(
                        ssa => new LibraryStudent_studentSchoolAssociation
                        {
                            entryGradeLevelDescriptor = ssa.entryGradeLevelDescriptor,
                            exitWithdrawDate = ssa.exitWithdrawDate,
                            schoolId = ssa.schoolReference.schoolId,
                            graduationSchoolYear = ssa.graduationPlanReference.graduationSchoolYear
                        }).ToList();

                // Get StudentSectionAssociation records
                var getStudentSectionAssociationsResponse =
                    _studentSectionAssociationsApi.GetStudentSectionAssociationsAll(
                        null, null,
                        studentResult.studentUniqueId,
                        null, null, null, null, null, null, null, null,
                        null, null, null, null, null, null, null);
                if (!ManualCompositesHelper.IsValidRestResponse(getStudentSectionAssociationsResponse))
                    return ManualCompositesHelper.CopyRestResponse(getStudentSectionAssociationsResponse, results);
                studentResult.sections =
                    getStudentSectionAssociationsResponse.Data.Select(
                        ssa => new LibraryStudent_studentSectionAssociation
                        {
                            homeroomIndicator = ssa.homeroomIndicator,
                            schoolId = ssa.sectionReference.schoolId,
                            classPeriodName = ssa.sectionReference.classPeriodName,
                            localCourseCode = ssa.sectionReference.localCourseCode,
                            termDescriptor = ssa.sectionReference.termDescriptor,
                            schoolYear = ssa.sectionReference.schoolYear,
                            classroomIdentificationCode = ssa.sectionReference.classroomIdentificationCode,
                            uniqueSectionCode = ssa.sectionReference.uniqueSectionCode,
                            sequenceOfCourse = ssa.sectionReference.sequenceOfCourse
                        }).ToList();

                // Get StudentLibraryService records
                var getStudentLibraryServicesResponse =
                    _studentLibraryServicesApi.GetStudentLibraryServicesAll(
                        null, null,
                        studentResult.studentUniqueId,
                        null, null, null, null, null, null, null, null,
                        null, null, null, null, null, null, null, null);
                if (!ManualCompositesHelper.IsValidRestResponse(getStudentLibraryServicesResponse))
                    return ManualCompositesHelper.CopyRestResponse(getStudentLibraryServicesResponse, results);

                studentResult.studentLibraryServices =
                    getStudentLibraryServicesResponse.Data.Select(
                        ssa => new LibraryStudent_studentLibraryService
                        {
                            barcode = ssa.barcode,
                            schoolId = ssa.schoolReference.schoolId,
                            accessLevelDescriptor = ssa.accessLevelDescriptor,
                            cardExpirationDate = ssa.cardExpirationDate,
                            isAcceptableUsePolicyOnFile = ssa.isAcceptableUsePolicyOnFile,
                            userName = ssa.userName,
                            password = ssa.password,
                            userDefined1 = ssa.userDefined1,
                            userDefined2 = ssa.userDefined2,
                            userDefined3 = ssa.userDefined3,
                            userDefined4 = ssa.userDefined4,
                            userDefined5 = ssa.userDefined5,
                            other1 = ssa.other1,
                            other2 = ssa.other2,
                            other3 = ssa.other3
                        }).ToList();
            }
            return ManualCompositesHelper.CopyRestResponse(originalRestResponse, studentResults);
        }

        /// <summary>
        ///     Retrieves resources based with paging capabilities (using the &quot;Get All&quot; pattern). This GET operation
        ///     provides access to resources using the &quot;Get All&quot; pattern. In this version of the API there is support for
        ///     paging.
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="fields">
        ///     Specifies a subset of properties that should be returned for each entity (e.g. &quot;
        ///     property1,collection1(collProp1,collProp2)&quot;).
        /// </param>
        /// <returns></returns>
        public IRestResponse<List<LibraryStaff>> GetLibraryStaffsAll(int? offset, int? limit, string fields)
        {
            var staffGetAllResponse = _staffApi.GetStaffsAll(offset, limit);

            if (!ManualCompositesHelper.IsValidRestResponse(staffGetAllResponse))
                return ManualCompositesHelper.CopyRestResponse(staffGetAllResponse, new List<LibraryStaff>());

            return GetManualLibraryStaffComposite(staffGetAllResponse.Data, staffGetAllResponse);
        }

        /// <summary>
        ///     Retrieves resources matching values of an example resource (using the &quot;Get By Example&quot; pattern). This GET
        ///     operation provides access to resources using the &quot;Get by Example&quot; search pattern.  The values of any
        ///     properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="staffUniqueId">A unique alpha-numeric code assigned to a staff.</param>
        /// <param name="personalTitlePrefix">A prefix used to denote the title, degree, position, or seniority of the person.</param>
        /// <param name="firstName">
        ///     A name given to an individual at birth, baptism, or during another naming ceremony, or through
        ///     legal change.
        /// </param>
        /// <param name="middleName">A secondary name given to an individual at birth, baptism, or during another naming ceremony.</param>
        /// <param name="lastSurname">The name borne in common by members of a family.</param>
        /// <param name="generationCodeSuffix">
        ///     An appendage, if any, used to denote an individual's generation in his family (e.g.,
        ///     Jr., Sr., III).
        /// </param>
        /// <param name="maidenName">The person's maiden name.</param>
        /// <param name="sexType">A person''s gender.</param>
        /// <param name="birthDate">The month, day, and year on which an individual was born.</param>
        /// <param name="hispanicLatinoEthnicity">
        ///     An indication that the individual traces his or her origin or descent to Mexico,
        ///     Puerto Rico, Cuba, Central, and South America, and other Spanish cultures, regardless of race. The term, &quot;
        ///     Spanish origin,&quot; can be used in addition to &quot;Hispanic or Latino.&quot;
        /// </param>
        /// <param name="oldEthnicityType">
        ///     Previous definition of Ethnicity combining Hispanic/latino and race:  1 - American
        ///     Indian or Alaskan Native  2 - Asian or Pacific Islander  3 - Black, not of Hispanic origin  4 - Hispanic  5 -
        ///     White, not of Hispanic origin
        /// </param>
        /// <param name="highestCompletedLevelOfEducationDescriptor">
        ///     A unique identifier used as Primary Key, not derived from
        ///     business logic, when acting as Foreign Key, references the parent table.
        /// </param>
        /// <param name="yearsOfPriorProfessionalExperience">
        ///     The total number of years that an individual has previously held a
        ///     similar professional position in one or more education institutions.
        /// </param>
        /// <param name="yearsOfPriorTeachingExperience">
        ///     The total number of years that an individual has previously held a
        ///     teaching position in one or more education institutions.
        /// </param>
        /// <param name="highlyQualifiedTeacher">
        ///     An indication of whether a teacher is classified as highly qualified for his/her
        ///     assignment according to state definition. This attribute indicates the teacher is highly qualified for ALL Sections
        ///     being taught.
        /// </param>
        /// <param name="loginId">The login ID for the user; used for security access control interface.</param>
        /// <param name="citizenshipStatusType">
        ///     A unique identifier used as Primary Key, not derived from business logic, when
        ///     acting as Foreign Key, references the parent table.
        /// </param>
        /// <param name="id"></param>
        /// <param name="educatorEffectivenessDescriptor">
        ///     (MI EXT) A unique identifier used as Primary Key, not derived from
        ///     business logic, when acting as Foreign Key, references the parent table.
        /// </param>
        /// <param name="newTeacherClassroomManagementProfessionalDevelopment">
        ///     (MI EXT) The number of days of professional
        ///     development in classroom management that was provided by the district for new teacher.
        /// </param>
        /// <param name="newTeacherInstructionalDeliveryProfessionalDevelopment">
        ///     (MI EXT) The number of days of professional
        ///     development in instructional deliver (strategies) that was provided by the district for new teacher.
        /// </param>
        /// <returns></returns>
        public IRestResponse<List<LibraryStaff>> GetLibraryStaffsByExample(int? offset, int? limit, string fields,
            string q, string staffUniqueId, string personalTitlePrefix, string firstName, string middleName,
            string lastSurname, string generationCodeSuffix, string maidenName, string sexType, DateTime? birthDate,
            bool? hispanicLatinoEthnicity, string oldEthnicityType, string highestCompletedLevelOfEducationDescriptor,
            double? yearsOfPriorProfessionalExperience, double? yearsOfPriorTeachingExperience,
            bool? highlyQualifiedTeacher, string loginId, string citizenshipStatusType, string id,
            string educatorEffectivenessDescriptor, double? newTeacherClassroomManagementProfessionalDevelopment,
            double? newTeacherInstructionalDeliveryProfessionalDevelopment)
        {
            var staffGetAllResponse = _staffApi.GetStaffsAll(offset, limit, staffUniqueId, personalTitlePrefix,
                firstName, middleName, lastSurname, generationCodeSuffix, maidenName, sexType, birthDate,
                hispanicLatinoEthnicity, oldEthnicityType, highestCompletedLevelOfEducationDescriptor,
                yearsOfPriorProfessionalExperience, yearsOfPriorTeachingExperience, highlyQualifiedTeacher,
                loginId, citizenshipStatusType, id, educatorEffectivenessDescriptor,
                newTeacherClassroomManagementProfessionalDevelopment,
                newTeacherInstructionalDeliveryProfessionalDevelopment);

            if (!ManualCompositesHelper.IsValidRestResponse(staffGetAllResponse))
                return ManualCompositesHelper.CopyRestResponse(staffGetAllResponse, new List<LibraryStaff>());

            return GetManualLibraryStaffComposite(staffGetAllResponse.Data, staffGetAllResponse);
        }

        /// <summary>
        ///     Retrieves a specific resource using the resource's identifier (using the &quot;Get By Key&quot; pattern). This GET
        ///     operation provides access to resources using the &quot;Get By Key&quot; pattern. The values of the natural key of
        ///     the resource must be fully specified, and the service will return the matching result (if it exists).
        /// </summary>
        /// <param name="staffUniqueId">A unique alphanumeric code assigned to a staff.</param>
        /// <param name="fields">
        ///     Specifies a subset of properties that should be returned for each entity (e.g. &quot;
        ///     property1,collection1(collProp1,collProp2)&quot;).
        /// </param>
        /// <param name="IfNoneMatch">
        ///     The previously returned ETag header value, used here to prevent the unnecessary data transfer
        ///     of an unchanged resource.
        /// </param>
        /// <returns></returns>
        public IRestResponse<LibraryStaff> GetLibraryStaffsByKey(string staffUniqueId, string fields,
            string IfNoneMatch)
        {
            var staffGetByKeyResponse = _staffApi.GetStaffByKey(staffUniqueId, IfNoneMatch);

            if (!ManualCompositesHelper.IsValidRestResponse(staffGetByKeyResponse))
                return ManualCompositesHelper.CopyRestResponse<LibraryStaff>(staffGetByKeyResponse, null);

            var mappedResponse = GetManualLibraryStaffComposite(new List<Staff> {staffGetByKeyResponse.Data},
                staffGetByKeyResponse);
            return ManualCompositesHelper.CopyRestResponse(mappedResponse, mappedResponse.Data.Single());
        }

        /// <summary>
        ///     Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET
        ///     operation provides access to resources using the &quot;Get By Id&quot; pattern.
        /// </summary>
        /// <param name="id"></param>
        /// <param name="fields">
        ///     Specifies a subset of properties that should be returned for each entity (e.g. &quot;
        ///     property1,collection1(collProp1,collProp2)&quot;).
        /// </param>
        /// <param name="IfNoneMatch">
        ///     The previously returned ETag header value, used here to prevent the unnecessary data transfer
        ///     of an unchanged resource.
        /// </param>
        /// <returns></returns>
        public IRestResponse<LibraryStaff> GetLibraryStaffsById(string id, string fields, string IfNoneMatch)
        {
            var staffByIdResponse = _staffApi.GetStaffsById(id, IfNoneMatch);

            if (!ManualCompositesHelper.IsValidRestResponse(staffByIdResponse))
                return ManualCompositesHelper.CopyRestResponse<LibraryStaff>(staffByIdResponse, null);

            var mappedResponse = GetManualLibraryStaffComposite(new List<Staff> {staffByIdResponse.Data},
                staffByIdResponse);
            return ManualCompositesHelper.CopyRestResponse(mappedResponse, mappedResponse.Data.Single());
        }

        private IRestResponse<List<LibraryStaff>> GetManualLibraryStaffComposite(List<Staff> staff,
            IRestResponse originalRestResponse)
        {
            var results = new List<LibraryStaff>();
            var staffResults = staff.Select(s =>
                new LibraryStaff
                {
                    staffUniqueId = s.staffUniqueId,
                    firstName = s.firstName,
                    middleName = s.middleName,
                    lastSurname = s.lastSurname,
                    sexType = s.sexType,
                    birthDate = s.birthDate,
                    otherNames = s.otherNames.Select(n =>
                        new LibraryStaff_staffOtherName
                        {
                            firstName = n.firstName,
                            middleName = n.middleName,
                            lastSurname = n.lastSurname,
                            otherNameType = n.otherNameType
                        }).ToList(),
                    addresses = s.addresses.Select(a =>
                        new LibraryStaff_staffAddress
                        {
                            addressType = a.addressType,
                            streetNumberName = a.streetNumberName,
                            apartmentRoomSuiteNumber = a.apartmentRoomSuiteNumber,
                            buildingSiteNumber = a.buildingSiteNumber,
                            city = a.city,
                            stateAbbreviationType = a.stateAbbreviationType,
                            postalCode = a.postalCode
                        }).ToList(),
                    electronicMails = s.electronicMails.Select(em =>
                        new LibraryStaff_staffElectronicMail
                        {
                            primaryEmailAddressIndicator = em.primaryEmailAddressIndicator,
                            electronicMailType = em.electronicMailType,
                            electronicMailAddress = em.electronicMailAddress
                        }).ToList(),
                    telephones = s.telephones.Select(t =>
                        new LibraryStaff_staffTelephone
                        {
                            telephoneNumberType = t.telephoneNumberType,
                            orderOfPriority = t.orderOfPriority,
                            telephoneNumber = t.telephoneNumber
                        }).ToList()
                }).ToList();
            foreach (var staffResult in staffResults)
            {
                var getStaffEducationOrganizationEmploymentAssociationsResponse =
                    _staffEducationOrganizationEmploymentAssociationsApi
                        .GetStaffEducationOrganizationEmploymentAssociationsAll(
                            null, null,
                            null, staffResult.staffUniqueId,
                            null, null, null, null, null, null, null, null, null, null,
                            null, null);
                if (
                    !ManualCompositesHelper.IsValidRestResponse(
                        getStaffEducationOrganizationEmploymentAssociationsResponse))
                    return
                        ManualCompositesHelper.CopyRestResponse(
                            getStaffEducationOrganizationEmploymentAssociationsResponse, results);
                staffResult.schools =
                    getStaffEducationOrganizationEmploymentAssociationsResponse.Data.Select(
                        seoea => new LibraryStaff_staffEducationOrganizationEmploymentAssociation
                        {
                            endDate = seoea.endDate,
                            educationOrganizationId = seoea.educationOrganizationReference.educationOrganizationId
                        }).ToList();

                // Get StaffLibraryService records
                var getStaffLibraryServicesResponse =
                    _staffLibraryServicesApi.GetStaffLibraryServicesAll(
                        null, null,
                        staffResult.staffUniqueId,
                        null, null, null, null, null, null, null, null,
                        null, null, null, null, null, null, null, null);
                if (!ManualCompositesHelper.IsValidRestResponse(getStaffLibraryServicesResponse))
                    return ManualCompositesHelper.CopyRestResponse(getStaffLibraryServicesResponse, results);

                staffResult.staffLibraryServices =
                    getStaffLibraryServicesResponse.Data.Select(
                        ssa => new LibraryStaff_staffLibraryService
                        {
                            barcode = ssa.barcode,
                            schoolId = ssa.schoolReference.schoolId,
                            accessLevelDescriptor = ssa.accessLevelDescriptor,
                            cardExpirationDate = ssa.cardExpirationDate,
                            isAcceptableUsePolicyOnFile = ssa.isAcceptableUsePolicyOnFile,
                            userName = ssa.userName,
                            password = ssa.password,
                            userDefined1 = ssa.userDefined1,
                            userDefined2 = ssa.userDefined2,
                            userDefined3 = ssa.userDefined3,
                            userDefined4 = ssa.userDefined4,
                            userDefined5 = ssa.userDefined5,
                            other1 = ssa.other1,
                            other2 = ssa.other2,
                            other3 = ssa.other3
                        }).ToList();
            }
            return ManualCompositesHelper.CopyRestResponse(originalRestResponse, staffResults);
        }
    }
}
