using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Models {
  public class Assessment {
    /* The unique identifier of the resource. */
    public string id { get; set; }

    /* A reference to the related AssessmentFamily resource. */
    public AssessmentFamilyReference assessmentFamilyReference { get; set; }

    /* The title or name of the assessment.  NEDM: Assessment Title */
    public string title { get; set; }

    /* The typical grade level for which an assessment is designed. If the test assessment spans a range of grades, then this attribute holds the highest grade assessed.  If only one grade level is assessed, then only this attribute is used. For example:  Adult  Prekindergarten  First grade  Second grade  ... */
    public string assessedGradeLevelDescriptor { get; set; }

    /* The description of the content or subject area (e.g., arts, mathematics, reading, stenography, or a foreign language) of an assessment.  NEDM: Assessment Content, Academic Subject */
    public string academicSubjectDescriptor { get; set; }

    /* The version identifier for the assessment. */
    public int? version { get; set; }

    /* A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string categoryDescriptor { get; set; }

    /* If the test assessment spans a range of grades, then this attribute holds the lowest grade assessed.  If only one grade level is assessed, then this attribute is omitted. For example:  Adult  Prekindergarten  First grade  Second grade  ... */
    public string lowestAssessedGradeLevelDescriptor { get; set; }

    /* Identifies the form of the assessment, for example a regular versus makeup form, multiple choice versus constructed response, etc. */
    public string form { get; set; }

    /* The month, day, and year that the conceptual design for the assessment was most recently revised substantially. */
    public DateTime? revisionDate { get; set; }

    /* The maximum raw score achievable across all assessment items that are correct and scored at the maximum. */
    public int? maxRawScore { get; set; }

    /* Reflects the common nomenclature for an element. */
    public string nomenclature { get; set; }

    /* The ID of the Assessment Period Descriptor */
    public string periodDescriptor { get; set; }

    /* Namespace for the Assessment. */
    public string @namespace { get; set; }

    /* An indication as to whether an assessment conforms to a standard. */
    public AssessmentContentStandard contentStandard { get; set; }

    /* An unordered collection of assessmentIdentificationCodes.  A unique number or alphanumeric code assigned to an assessment by a school, school system, a state, or other agency or entity. */
    public List<AssessmentIdentificationCode> identificationCodes { get; set; }

    /* An unordered collection of assessmentLanguages.  An indication of the languages in which the Assessment is designed. */
    public List<AssessmentLanguage> languages { get; set; }

    /* An unordered collection of assessmentPerformanceLevels.  Definition of the performance levels and the associated cut scores. Three styles are supported: 1. Specification of performance level by minimum and maximum score 2. Specification of performance level by cut score, using only minimum score 3. Specification of performance level without any mapping to scores . */
    public List<AssessmentPerformanceLevel> performanceLevels { get; set; }

    /* An unordered collection of assessmentPrograms.  The programs associated with the Assessment. */
    public List<AssessmentProgram> programs { get; set; }

    /* An unordered collection of assessmentScores.  Definition of the scores to be expected from this assessment. */
    public List<AssessmentScore> scores { get; set; }

    /* An unordered collection of assessmentSections.  The section(s) to which the Assessment is associated. */
    public List<AssessmentSection> sections { get; set; }

    /* A unique system-generated value that identifies the version of the resource. */
    public string _etag { get; set; }

    }
}
