using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Models {
  public class AssessmentFamily {
    /* The unique identifier of the resource. */
    public string id { get; set; }

    /* A reference to the related AssessmentFamily resource. */
    public AssessmentFamilyReference parentAssessmentFamilyReference { get; set; }

    /* The title or name of the assessment family. */
    public string title { get; set; }

    /* A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string assessmentCategoryDescriptor { get; set; }

    /* Key for AcademicSubject */
    public string academicSubjectDescriptor { get; set; }

    /* Key for GradeLevel */
    public string assessedGradeLevelDescriptor { get; set; }

    /* Key for GradeLevel */
    public string lowestAssessedGradeLevelDescriptor { get; set; }

    /* The version identifier for the assessment. */
    public int? version { get; set; }

    /* The month, day, and year that the conceptual design for the AssessmentFamily was most recently revised substantially. */
    public DateTime? revisionDate { get; set; }

    /* Reflects the common nomenclature for an element. */
    public string nomenclature { get; set; }

    /* Namespace for the Assessments in this AssessmentFamily. */
    public string @namespace { get; set; }

    /* An indication as to whether an assessment conforms to a standard (e.g., local standard, statewide standard, regional standard, association standard). */
    public AssessmentFamilyContentStandard contentStandard { get; set; }

    /* An unordered collection of assessmentFamilyAssessmentPeriods.  The periods or windows defined in which an assessment is supposed to be administered. */
    public List<AssessmentFamilyAssessmentPeriod> assessmentPeriods { get; set; }

    /* An unordered collection of assessmentFamilyIdentificationCodes.  A unique number or alphanumeric code assigned to an assessment family by a school, school system, a state, or other agency or entity. */
    public List<AssessmentFamilyIdentificationCode> identificationCodes { get; set; }

    /* An unordered collection of assessmentFamilyLanguages.  An indication of the languages in which the Assessment Family is designed. */
    public List<AssessmentFamilyLanguage> languages { get; set; }

    /* A unique system-generated value that identifies the version of the resource. */
    public string _etag { get; set; }

    }
}
