using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Models {
  public class EducationContent {
    /* The unique identifier of the resource. */
    public string id { get; set; }

    /* A reference to the related LearningStandard resource. */
    public LearningStandardReference learningStandardReference { get; set; }

    /* The identifier of the content standard. */
    public string contentIdentifier { get; set; }

    /* The public web site address (URL), file, or ftp locator. */
    public string learningResourceMetadataURI { get; set; }

    /* A shortened description for reference. */
    public string shortDescription { get; set; }

    /* A detailed description of the entity. */
    public string description { get; set; }

    /* Indicates whether there are additional un-named authors. In a research report, this is often marked by the abbreviation "et al". */
    public bool? additionalAuthorsIndicator { get; set; }

    /* The organization credited with publishing the resource. */
    public string publisher { get; set; }

    /* Approximate or typical time it takes to work with or through this learning resource for the typical intended target audience. */
    public string timeRequired { get; set; }

    /* A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string interactivityStyleType { get; set; }

    /* A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string contentClassType { get; set; }

    /* The URL where the owner specifies permissions for using the resource. */
    public string useRightsURL { get; set; }

    /* The date on which this content was first published. */
    public DateTime? publicationDate { get; set; }

    /* The year at which this content was first published. */
    public int? publicationYear { get; set; }

    /* The version identifier for the content. */
    public string version { get; set; }

    /* An amount that has to be paid or spent to buy or obtain the EducationContent. */
    public double? cost { get; set; }

    /* A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table. */
    public string costRateType { get; set; }

    /* Namespace for the EducationContent. */
    public string @namespace { get; set; }

    /* An unordered collection of educationContentAppropriateGradeLevels.  Grade levels for which this education content is applicable-if omitted, considered generally applicable. */
    public List<EducationContentAppropriateGradeLevel> appropriateGradeLevels { get; set; }

    /* An unordered collection of educationContentAppropriateSexes.  Genders for which this education content is applicable-if omitted, considered generally applicable. */
    public List<EducationContentAppropriateSex> appropriateSexes { get; set; }

    /* An unordered collection of educationContentAuthors.  The individual credited with the creation of the resource. */
    public List<EducationContentAuthor> authors { get; set; }

    /* An unordered collection of educationContentDerivativeSourceEducationContents.  A reference to the Education Content from which this Education Content was derived. */
    public List<EducationContentDerivativeSourceEducationContent> derivativeSourceEducationContents { get; set; }

    /* An unordered collection of educationContentDerivativeSourceLearningResourceMetadataURIs.  A reference to the Learning Resource Metadata URI from which this Education Content was derived. */
    public List<EducationContentDerivativeSourceLearningResourceMetadataURI> derivativeSourceLearningResourceMetadataURIs { get; set; }

    /* An unordered collection of educationContentDerivativeSourceURIs.  A reference to the URI from which this Education Content was derived. */
    public List<EducationContentDerivativeSourceURI> derivativeSourceURIs { get; set; }

    /* An unordered collection of educationContentLanguages.  An indication of the languages in which the Education Content is designed. */
    public List<EducationContentLanguage> languages { get; set; }

    /* A unique system-generated value that identifies the version of the resource. */
    public string _etag { get; set; }

    }
}
